<?php

declare(strict_types=1);

namespace Psl\IO;

use Psl\IO;

interface CloseSeekStreamHandleInterface extends
    CloseStreamHandleInterface,
    IO\CloseSeekHandleInterface,
    SeekStreamHandleInterface
{
}
