<?php

declare(strict_types=1);

namespace Psl\Math\Exception;

use Psl\Exception\InvalidArgumentException;

class ArithmeticException extends InvalidArgumentException implements ExceptionInterface
{
}
